@@ -1,3 +1,5 @@
+# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
+
 import httplib
 import json
 from pprint import pprint
@@ -6,11 +8,14 @@ from fifo.helper import *
 class Wiggle:
     def __init__(self):
         self._token = False
-        self
-    def init(self, host, user, pw, token):
-	self.host = host
+        self._apiEndpoint = "/api/0.1.0/"
+
+    def init(self, host, user, pw, token, apiVersion):
+        self.host = host
         self.headers = {"Content-type": "application/json;charset=UTF-8",
-                        "Accept": "application/json"}
+                "Accept": "application/json"}
+        if apiVersion:
+            self._apiEndpoint = "/api/" + apiVersion + "/"
         if token:
             if not self.set_token(token):
                 self.connect(user, pw)
@@ -18,7 +23,7 @@ class Wiggle:
             self.connect(user, pw)
 
     def conn(self):
-	return httplib.HTTPSConnection(self.host)
+        return httplib.HTTPSConnection(self.host)
 
     def get_token(self):
         return self._token
@@ -28,19 +33,29 @@ class Wiggle:
         self.headers["X-Snarl-Token"] = self._token
         return self.get("sessions", token)
 
-    def get(self, resource, entity):
+    def get(self, resource, entity, suffix="", rawResponse=False, headers=None):
         conn = self.conn()
-	conn.request("GET", "/api/0.1.0/" + resource + "/" + entity, "", self.headers)
+        hdrs = headers or self.headers
+        vprint("GET", self._apiEndpoint + resource + "/" + entity + suffix, "", hdrs)
+        curlprint(self.host, "GET", self._apiEndpoint + resource + "/" + entity + suffix, hdrs)
+        conn.request("GET", self._apiEndpoint + resource + "/" + entity + suffix, "", hdrs)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status != 200):
             return False
         else:
-            return json.loads(response.read())
+            if rawResponse:
+                return response
+            else:
+                return json.loads(response.read())
 
     def delete(self, resource, entity):
         conn = self.conn()
-        conn.request("DELETE", "/api/0.1.0/" + resource + "/" + entity, "", self.headers)
+        vprint("DELETE", self._apiEndpoint + resource + "/" + entity, "", self.headers)
+        curlprint(self.host, "DELETE", self._apiEndpoint + resource + "/" + entity, self.headers)
+        conn.request("DELETE", self._apiEndpoint + resource + "/" + entity, "", self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status >= 200 and response.status < 300):
             return True
         else:
@@ -48,8 +63,12 @@ class Wiggle:
 
     def delete_body(self, resource, entity, body):
         conn = self.conn()
-        conn.request("DELETE", "/api/0.1.0/" + resource + "/" + entity, json.dumps(body), self.headers)
+        jbody = json.dumps(body)
+        vprint("DELETE", self._apiEndpoint + resource + "/" + entity, jbody, self.headers)
+        curlprint(self.host, "DELETE", self._apiEndpoint + resource + "/" + entity, headers=self.headers, data=jbody)
+        conn.request("DELETE", self._apiEndpoint + resource + "/" + entity, jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status >= 200 and response.status < 300):
             return True
         else:
@@ -59,9 +78,12 @@ class Wiggle:
         conn = self.conn()
         if isinstance(attr, str):
             attr = [attr]
-        url = "/api/0.1.0/" + resource + "/" + entity + "/" + "/".join(attr)
+        url = self._apiEndpoint + resource + "/" + entity + "/" + "/".join(attr)
+        vprint("DELETE", url, "", self.headers)
+        curlprint(self.host, "DELETE", url, headers=self.headers)
         conn.request("DELETE", url, "", self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status < 300 and response.status >= 300):
             return False
         else:
@@ -71,9 +93,13 @@ class Wiggle:
         conn = self.conn()
         if isinstance(attr, str):
             attr = [attr]
-        url = "/api/0.1.0/" + resource + "/" + entity + "/" + "/".join(attr)
-        conn.request("DELETE", url, json.dumps(body), self.headers)
+        url = self._apiEndpoint + resource + "/" + entity + "/" + "/".join(attr)
+        jbody = json.dumps(body)
+        vprint("DELETE", url, jbody, self.headers)
+        curlprint(self.host, "DELETE", url, headers=self.headers, data=jbody)
+        conn.request("DELETE", url, jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status < 300 and response.status >= 300):
             return False
         else:
@@ -83,9 +109,13 @@ class Wiggle:
         conn = self.conn()
         if isinstance(attr, str):
             attr = [attr]
-        url = "/api/0.1.0/" + resource + "/" + entity + "/" + "/".join(attr)
-        conn.request("PUT", url, json.dumps(body), self.headers)
+        jbody = json.dumps(body)
+        vprint("PUT", url, jbody, self.headers)
+        curlprint(self.host, "PUT", url, headers=self.headers, data=jbody)
+        url = self._apiEndpoint + resource + "/" + entity + "/" + "/".join(attr)
+        conn.request("PUT", url, jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status < 300 and response.status >= 300):
             return False
         else:
@@ -95,9 +125,13 @@ class Wiggle:
         conn = self.conn()
         if isinstance(attr, str):
             attr = [attr]
-        url = "/api/0.1.0/" + resource + "/" + entity + "/" + "/".join(attr)
-        conn.request("POST", url, json.dumps(body), self.headers)
+        url = self._apiEndpoint + resource + "/" + entity + "/" + "/".join(attr)
+        jbody = json.dumps(body)
+        vprint("POST", url, jbody, self.headers)
+        curlprint(self.host, "POST", url, headers=self.headers, data=jbody)
+        conn.request("POST", url, jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status == 303):
             newurl = response.getheader('Location')
             conn = self.conn()
@@ -107,6 +141,8 @@ class Wiggle:
                 return False
             else:
                 return json.loads(response.read())
+        elif (response.status == 201):
+            return 201
         elif (response.status == 200):
             return json.loads(response.read())
         else:
@@ -116,9 +152,13 @@ class Wiggle:
         conn = self.conn()
         if isinstance(attr, str):
             attr = [attr]
-        url = "/api/0.1.0/" + resource + "/" + entity + "/" + "/".join(attr)
+        url = self._apiEndpoint + resource + "/" + entity + "/" + "/".join(attr)
+        jbody = json.dumps(body)
+        vprint("GET", url, "", self.headers)
+        curlprint(self.host, "GET", url, headers=self.headers)
         conn.request("GET", url, "", self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status != 200):
             return False
         else:
@@ -128,9 +168,31 @@ class Wiggle:
         conn = self.conn()
         if isinstance(entity, str):
             entity = [entity]
-        url = "/api/0.1.0/" + resource + "/" + "/".join(entity)
-        conn.request("PUT", url, json.dumps(body), self.headers)
+        url = self._apiEndpoint + resource + "/" + "/".join(entity)
+        jbody = json.dumps(body)
+        vprint("PUT", url, jbody, self.headers)
+        curlprint(self.host, "PUT", url, headers=self.headers, data=jbody)
+        conn.request("PUT", url, jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
+        if (response.status != 200):
+            return False
+        else:
+            return json.loads(response.read())
+
+    def put_bindata(self, resource, entity, file, suffix="", headers=None):
+        conn = self.conn()
+        if isinstance(entity, str):
+            entity = [entity]
+        url = self._apiEndpoint + resource + "/" + "/".join(entity) + suffix
+        hdrs = headers or self.headers
+        vprint("PUT", url, "@" + file.name, hdrs)
+        curlprint(self.host, "PUT", url, headers=hdrs, upload=file.name)
+        conn.request("PUT", url, file, hdrs)
+        response = conn.getresponse()
+        vprint("Status: ", response.status)
+        if (response.status > 200 and response.status > 200):
+            return response.status
         if (response.status != 200):
             return False
         else:
@@ -140,9 +202,40 @@ class Wiggle:
         conn = self.conn()
         if isinstance(resource, str):
             resource = [resource]
-        url = "/api/0.1.0/" + "/".join(resource)
-        conn.request("POST", url, json.dumps(body), self.headers)
+        url = self._apiEndpoint + "/".join(resource)
+        jbody = json.dumps(body)
+        vprint("POST", url, jbody, self.headers)
+        curlprint(self.host, "POST", url, headers=self.headers, data=jbody)
+        conn.request("POST", url, jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
+        if (response.status == 303):
+            newurl = response.getheader('Location')
+            conn = self.conn()
+            conn.request("GET", newurl, "", self.headers)
+            response = conn.getresponse()
+            if (response.status != 200):
+                print response.status
+                return False
+            else:
+                return json.loads(response.read())
+        elif (response.status == 200):
+            return json.loads(response.read())
+        else:
+            print response.status
+            return False
+
+    def post_bindata(self, resource, entity, file, headers=None):
+        conn = self.conn()
+        if isinstance(resource, str):
+            resource = [resource]
+        url = self._apiEndpoint + "/".join(resource) + "/" + entity
+        hdrs = headers or self.headers
+        vprint("POST", url, "@" + file.name, hdrs)
+        curlprint(self.host, "POST", url, headers=hdrs, file=file.name, fileMode="binary")
+        conn.request("POST", url, file, hdrs)
+        response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status == 303):
             newurl = response.getheader('Location')
             conn = self.conn()
@@ -160,31 +253,41 @@ class Wiggle:
             return False
 
     def list(self, resource):
-	conn = self.conn();
-        conn.request("GET", "/api/0.1.0/" + resource, "", self.headers)
+        conn = self.conn()
+        vprint("GET", self._apiEndpoint + resource, "", self.headers)
+        curlprint(self.host, "GET", self._apiEndpoint + resource, headers=self.headers)
+        conn.request("GET", self._apiEndpoint + resource, "", self.headers)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status != 200):
             return False
         else:
             return json.loads(response.read())
 
     def full_list(self, resource, fields):
-	conn = self.conn();
-        hdrs = self.headers;
-        hdrs["x-full-list"] = "true";
+        conn = self.conn()
+        hdrs = self.headers
+        hdrs["x-full-list"] = "true"
         if fields != []:
-            hdrs["x-full-list-fields"] = ",".join(fields);
-        conn.request("GET", "/api/0.1.0/" + resource, "", hdrs)
+            hdrs["x-full-list-fields"] = ",".join(fields)
+        vprint("GET", self._apiEndpoint + resource, "", hdrs)
+        curlprint(self.host, "GET", self._apiEndpoint + resource, headers=hdrs)
+        conn.request("GET", self._apiEndpoint + resource, "", hdrs)
         response = conn.getresponse()
+        vprint("Status: ", response.status)
         if (response.status != 200):
             return False
         else:
             return json.loads(response.read())
 
     def connect(self, user, pw):
-	conn = self.conn();
-        conn.request("POST", "/api/0.1.0/sessions",  json.dumps({"user":user, "password": pw}), self.headers)
+        conn = self.conn()
+        jbody = json.dumps({"user":user, "password": pw})
+        vprint("POST", self._apiEndpoint + "sessions",  jbody, self.headers)
+        curlprint(self.host, "POST", self._apiEndpoint + "sessions", headers=self.headers, data=jbody)
+        conn.request("POST", self._apiEndpoint + "sessions",  jbody, self.headers)
         response = conn.getresponse()
+        vprint("Status:", response.status)
         if (response.status == 303):
             self._token = response.getheader("X-Snarl-Token")
             self.headers["X-Snarl-Token"] = self._token
@@ -192,7 +295,6 @@ class Wiggle:
         else:
             return False
 
-
 class Cloud:
     def __init__(self, wiggle):
         self._wiggle = wiggle
@@ -212,7 +314,15 @@ class Entity:
         return self._wiggle.put(self._resource, uuid, body)
 
     def _post(self, body):
-        return self._wiggle.post(self._resource, body)
+        return self._wiggle.post(self._resource, body=body)
+
+    def _put_file(self, entity, target, file):
+        hdrs = dict(self._wiggle.headers)
+        hdrs["Content-Type"] = "application/x-gzip"
+        return self._wiggle.put_bindata(self._resource, entity, headers=hdrs, suffix=target, file=file)
+
+    def _post_file(self, entity, file):
+        return self._wiggle.post_bindata(self._resource, entity, file=file)
 
     def _post_attr(self, uuid, entity, body):
         uuid = self.uuid_by_name(uuid)
@@ -226,7 +336,6 @@ class Entity:
         uuid = self.uuid_by_name(uuid)
         return self._wiggle.delete_attr(self._resource, uuid, entity)
 
-
     def _delete_attr_body(self, uuid, entity, body):
         uuid = self.uuid_by_name(uuid)
         return self._wiggle.delete_attr_body(self._resource, uuid, entity, body)
@@ -268,6 +377,15 @@ class Entity:
         uuid = self.uuid_by_name(uuid)
         return self._wiggle.delete_body(self._resource, uuid, body)
 
+    def get_bindata(self, uuid, target, accept=""):
+        uuid = self.uuid_by_name(uuid)
+        binHeaders = dict(self._wiggle.headers)
+        if accept:
+            binHeaders["Accept"] = accept
+        else:
+            del binHeaders["Accept"]
+        return self._wiggle.get(self._resource, uuid, suffix=target, rawResponse=True, headers=binHeaders)
+
     def get_metadata(self, uuid):
         uuid = self.uuid_by_name(uuid)
         return self._wiggle.get_attr(self._resource, uuid, "metadata")
@@ -304,7 +422,6 @@ class Entity:
         parser_mdata_del.add_argument("key", help="key of the metadata")
 
 
-
 def mdata_get(args):
     e = args.endpoint.get(args.uuid)
     if not e or not e.has_key('metadata'):
